//+------------------------------------------------------------------+
//|                                                     MACD_CCI.mq4 |
//+------------------------------------------------------------------+

#property  indicator_separate_window
#property  indicator_buffers 3
#property  indicator_color1  RoyalBlue
#property  indicator_color2  Red
#property  indicator_color3  Gold
#property  indicator_minimum 0
#property  indicator_maximum 77
#property  indicator_width1  2
#property  indicator_width2  2
#property  indicator_width3  2

//
//
//
//
//

extern string TimeFrame  = "current time frame";
extern int    macdMAfast = 144;
extern int    macdMAslow = 233;
extern int    macdSigMA  =  21;
extern int    macdPrice  =   0;
extern int    macdLnMode =   0;
extern int    CCIperiod  =  50;
extern int    CCIprice   =   5;
extern double BarLevl    =   5;

//
//
//
//
//

extern string MA_Method_Price = "SMA0 EMA1 SMMA2 LWMA3||0O,1C 2H3L,4Md 5Tp 6WghC: Md(HL/2)4,Tp(HLC/3)5,Wgh(HLCC/4)6||macdLnMode:0main 1sig";

extern bool   alertsOn          = true;
extern bool   alertsOnCurrent   = true;
extern bool   alertsMessage     = true;
extern bool   alertsSound       = false;
extern bool   alertsEmail       = false;

//
//
//
//
//

double buffer1[];
double buffer2[];
double buffer3[];

int      timeFrame;
datetime lastABar;
bool     returningBars=false;
string   IndicatorFileName;
string   timeFrameDescription="current time frame";

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,buffer1); SetIndexStyle(0,DRAW_ARROW); SetIndexArrow(0,167); SetIndexEmptyValue(0,EMPTY_VALUE); 
   SetIndexBuffer(1,buffer2); SetIndexStyle(1,DRAW_ARROW); SetIndexArrow(1,167); SetIndexEmptyValue(1,EMPTY_VALUE);
   if (TimeFrame=="returnBars")
   {
      returningBars = true;
      return(0);
   }
   SetIndexBuffer(2,buffer3); SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,167); SetIndexEmptyValue(2,EMPTY_VALUE); 
      
   //
   //
   //
   //
   //
      
   timeFrame            = stringToTimeFrame(TimeFrame);
   timeFrameDescription = timeFrameToString(timeFrame);
   IndicatorFileName = WindowExpertName();

      string name1= " macd ("+macdMAfast+","+macdMAslow+","+macdSigMA+")";
      string name2= " cci (" +CCIperiod+")";
   IndicatorShortName("MACD-CCI "+timeFrameToString(timeFrame)+name1+name2);  
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars = IndicatorCounted();
   int i,limit;

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
      limit = MathMin(Bars - counted_bars,Bars-1);

      if (returningBars) { buffer1[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
      
   if (timeFrame > Period()) limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,IndicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for (i=limit; i>= 0; i--)
   {
      int y =iBarShift(NULL,timeFrame,Time[i]);

         double  macd0 = iMACD(NULL,timeFrame,macdMAfast,macdMAslow,macdSigMA,macdPrice,macdLnMode,y);
         double  macd1 = iMACD(NULL,timeFrame,macdMAfast,macdMAslow,macdSigMA,macdPrice,macdLnMode,y+1);
         double  cci   = iCCI (NULL,timeFrame,CCIperiod,CCIprice,y);
                
         buffer1[i]=EMPTY_VALUE;
         buffer2[i]=EMPTY_VALUE;
         buffer3[i]=EMPTY_VALUE;
      
         if (cci > 0 && macd0 > macd1) buffer1[i] = BarLevl;
         else 
            if (cci < 0 && macd0 < macd1) buffer2[i] = BarLevl;  
            else                          buffer3[i] = BarLevl;
   }
   
   //
   //
   //
   //
   //

   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));

      //
      //
      //
      //
      //
         
      if (buffer1[whichBar] != EMPTY_VALUE && buffer1[whichBar+1] == EMPTY_VALUE)
            doAlert(whichBar,timeFrameDescription+" change color to blue");
      if (buffer2[whichBar] != EMPTY_VALUE && buffer2[whichBar+1] == EMPTY_VALUE)
            doAlert(whichBar,timeFrameDescription+" change color to red");
   }
   return(0);
}



//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," Indicator MACD_CCI ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"Indicator MACD_CCI "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}